package backend

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"kpt-tmr-group/model"
	"kpt-tmr-group/pkg/logger/zaplog"
	"kpt-tmr-group/pkg/xerr"
	operationPb "kpt-tmr-group/proto/go/backend/operation"
	"net/http"
	"strconv"
	"strings"

	"github.com/xuri/excelize/v2"

	"go.uber.org/zap"
)

type PastureClientHandler func(ctx context.Context, pastureId int64, body interface{}) error

// type eventHandler func(ev map[string]interface{}, openID string, appID string, enterpriseID int, cts int64, conn redis.Conn) error

// PastureDetailById 获取指定牧场详情
func (s *StoreEntry) PastureDetailById(ctx context.Context, pastureId int64) (*model.GroupPasture, error) {
	result := &model.GroupPasture{Id: pastureId}
	if err := s.DB.Where("is_delete = ? and is_show = ?", operationPb.IsShow_OK, operationPb.IsShow_OK).First(result).Error; err != nil {
		return nil, xerr.WithStack(err)
	}
	return result, nil
}

func (s *StoreEntry) PastureHttpClient(ctx context.Context, apiUrl string, pastureId int64, body, response interface{}) error {
	pastureDetail, err := s.PastureDetailById(ctx, pastureId)
	if err != nil {
		zaplog.Error("SearchFormulaEstimateList", zap.Any("Err", err), zap.Int64("pastureId", pastureId))
		return xerr.Customf("该牧场数据错误,Err:%s", err)
	}

	pastureClient := model.NewPastureClient(pastureDetail)
	url := fmt.Sprintf("%s/%s", pastureDetail.Domain, apiUrl)

	result, err := pastureClient.DoPost(url, body)
	if err != nil {
		return xerr.WithStack(err)
	}

	zaplog.Info("PastureHttpClient", zap.String("url", url), zap.Any("request", body), zap.String("response", string(result)))
	if err = json.Unmarshal(result, response); err != nil {
		return xerr.WithStack(err)
	}
	return nil

}

// SearchFormulaEstimateList 配方评估
func (s *StoreEntry) SearchFormulaEstimateList(ctx context.Context, req *operationPb.SearchFormulaEstimateRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.FormulaEstimateParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.EndTime,
			Search:    fmt.Sprintf("%d", req.SearchType),
			TempletId: fmt.Sprintf("%d", req.TemplateId),
			Barid:     fmt.Sprintf("%d", req.BarnId),
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchInventoryStatistics 库存管理-库存统计
func (s *StoreEntry) SearchInventoryStatistics(ctx context.Context, req *operationPb.SearchInventoryStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.InventoryStatisticsParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.EndTime,
			FeedName:  req.FeedName,
		},
	}
	response := &model.PastureCommonResponse{
		Data: &model.PastureCommonData{
			List: make([]*model.InventoryStatisticsList, 0),
		},
	}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// InventoryStatisticsExcelExport 库存管理-库存统计-报表导出
func (s *StoreEntry) InventoryStatisticsExcelExport(ctx context.Context, req *operationPb.SearchInventoryStatisticsRequest) (*bytes.Buffer, error) {
	result, err := s.SearchInventoryStatistics(ctx, req)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	b, _ := json.Marshal(result.Data.List)
	inventoryStatisticsList := make([]*model.InventoryStatisticsList, 0)
	if err = json.Unmarshal(b, &inventoryStatisticsList); err != nil {
		return nil, xerr.Customf("牧场端返回数据错误")
	}

	file := excelize.NewFile()
	defer file.Close()

	streamWriter, err := file.NewStreamWriter(model.DefaultSheetName)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	// 表头
	titles := map[string][]interface{}{
		"A1": {"饲料名称", "期初", nil, "用料", nil, nil, nil, "期末"},
		"A2": {nil, "期初库存（kg）", "期初金额（元）", "入库重量（kg）", "系统出库重量（kg）", "人工用料重量（kg）", "损耗重量", "期末库存（kg）", "期末金额（kg）"},
	}
	for cell, values := range titles {
		if err = streamWriter.SetRow(cell, values); err != nil {
			return nil, xerr.WithStack(err)
		}
	}

	for i, item := range inventoryStatisticsList {
		cell, err := excelize.CoordinatesToCellName(1, i+3)
		if err != nil {
			zaplog.Error("InventoryStatisticsExcelExport CoordinatesToCellName", zap.Any("Err", err))
			continue
		}
		row := make([]interface{}, 0)
		row = append(row, item.FeedName, item.StartSum, item.StartPrice, item.LaidSum, item.UseSumXT, item.UseSumRG, item.UseSumXH, item.StopSum, item.StopPrice)

		if err = streamWriter.SetRow(cell, row); err != nil {
			return nil, xerr.WithStack(err)
		}
	}

	hvCell := map[string]string{
		"A1": "A2",
		"B1": "C1",
		"D1": "G1",
		"H1": "I1",
	}
	// 合并单元格
	for h, v := range hvCell {
		if err = streamWriter.MergeCell(h, v); err != nil {
			return nil, xerr.WithStack(err)
		}
	}

	// 修改样式
	/*style1, err := file.NewStyle(&excelize.Style{
		Fill:      excelize.Fill{Type: "pattern", Color: []string{"#DFEBF6"}, Pattern: 1},
		Alignment: &excelize.Alignment{Horizontal: "center"},
	})
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	style1, err := file.NewStyle(&excelize.Style{
		//Fill:      excelize.Fill{Type: "pattern", Color: []string{"#DFEBF6"}, Pattern: 1},
		Alignment: &excelize.Alignment{Horizontal: "center"},
	})
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	if err = file.SetCellStyle(model.DefaultSheetName, "A1", "A1", style1); err != nil {
		return nil, xerr.WithStack(err)
	}*/

	if err = streamWriter.Flush(); err != nil {
		return nil, xerr.WithStack(err)
	}

	return file.WriteToBuffer()
}

// SearchUserMaterialsStatistics 库存管理-用料分析
func (s *StoreEntry) SearchUserMaterialsStatistics(ctx context.Context, req *operationPb.SearchUserMaterialsStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		Checked:    req.ErrorCheck,
		ParamMaps: &model.UserMaterialsStatisticsParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.EndTime,
			FeedName:  req.FeedName,
			Typea:     fmt.Sprintf("%d", req.TypeCheck),
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{
		List: &model.UserMaterialsList{},
	}}
	if err := s.PastureHttpClient(ctx, model.UrlReportForm, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

func (s *StoreEntry) UserMaterialsStatisticsExcelExport(ctx context.Context, req *operationPb.SearchUserMaterialsStatisticsRequest) (*bytes.Buffer, error) {
	result, err := s.SearchUserMaterialsStatistics(ctx, req)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	b, _ := json.Marshal(result.Data.List)
	userMaterialsList := &model.UserMaterialsList{}
	if err = json.Unmarshal(b, userMaterialsList); err != nil {
		return nil, xerr.Customf("牧场端返回数据错误")
	}

	file := excelize.NewFile()
	defer file.Close()

	streamWriter, err := file.NewStreamWriter(model.DefaultSheetName)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	// 表头
	excelValuesList := map[string][]interface{}{}
	/*"A1": {
		"饲料名称", "需加料", nil, "Y国产燕麦草55", nil, "自制高产Ⅱ精补料1#", nil, "Y苜蓿M150", nil, "自制酵贮康1#", nil,
		"Y麸皮^大片", nil, "Y甜菜粕颗粒", nil, "3Y青贮玉米", nil, "围产牛补充预混合饲料1#", nil, "5Y青贮玉米", nil, "Y羊草", nil,
		"Y国产燕麦草60", nil, "Y苜蓿M170", nil, "Y苜蓿Z130", nil, "1Y青贮玉米", nil, "水", nil, "自制干奶精补料1#", nil,
		"自制初产精补料1#", nil, "自制高产精补料1#", nil, "自制低产精补料1#", nil, "Y青贮玉米^棒子", nil, "Y压片玉米", nil,
		"自制围产精补料1#", nil, "自制青年精补料1#", nil, "合计",
	},*/

	for _, data2 := range userMaterialsList.Data2 {
		excelValuesList["A1"] = append(excelValuesList["A1"], data2.Label)
		for _, c := range data2.Children {
			excelValuesList["A2"] = append(excelValuesList["A2"], c.Label)
		}
	}
	for _, data2 := range userMaterialsList.Data2 {
		for i, data1 := range userMaterialsList.Data1 {
			index := fmt.Sprintf("A%d", i+3)
			//value := tool.GetTargetByValueForTag(data1, c.Prop, "json")
			data1Map, ok := data1.(map[string]interface{})
			fmt.Println("======excelValuesList======", ok, data1Map, index, data2)

			// excelValuesList[index] = append(excelValuesList[index], data1Map[c.Prop])
		}
	}
	vv, _ := json.Marshal(excelValuesList)
	fmt.Println("======excelValuesList======", string(vv))
	/*for cell, values := range excelValuesList {
		if err = streamWriter.SetRow(cell, values); err != nil {
			return nil, xerr.WithStack(err)
		}
	}

	hvCell := map[string]string{
		"B1": "C1",
		"D1": "E1",
		"F1": "G1",
		"H1": "R1",
		"J1": "K1",
		"L1": "M1",
		"N1": "O1",
		"P1": "Q1",
		"R1": "S1",
		"T1": "U1",
		"V1": "W1",
		"Z1": "AA1",
	}
	// 合并单元格
	for h, v := range hvCell {
		if err = streamWriter.MergeCell(h, v); err != nil {
			return nil, xerr.WithStack(err)
		}
	}*/

	if err = streamWriter.Flush(); err != nil {
		return nil, xerr.WithStack(err)
	}

	return file.WriteToBuffer()
}

// SearchPriceStatistics 库存管理-价格分析
func (s *StoreEntry) SearchPriceStatistics(ctx context.Context, req *operationPb.SearchPriceStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.PriceStatisticsParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.EndTime,
			FeedName:  req.FeedName,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlReportForm, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchFeedStatistics 饲喂效率-效率统计
func (s *StoreEntry) SearchFeedStatistics(ctx context.Context, req *operationPb.SearchFeedStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.FeedStatisticsParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.StartTime,
			Date:      req.StartTime,
			FeedTName: req.FormulaTemplate,
			BarName:   req.BarnName,
			CowClass:  req.CattleCategoryName,
			Times:     fmt.Sprintf("%d", req.ClassNumber),
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// FeedChartStatistics 饲喂效率图表分析
func (s *StoreEntry) FeedChartStatistics(ctx context.Context, req *operationPb.FeedChartStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.FeedChartParams{
		ParamMaps: &model.ParamMaps{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.StartTime,
			Status:    req.Status,
		},
	}
	url, ok := model.UrlChart[req.ApiType]
	if !ok {
		return nil, xerr.Customf("错误的接口类型:%s", req.ApiType)
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, url, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// CowsAnalysis 饲喂效率-牛群评估
func (s *StoreEntry) CowsAnalysis(ctx context.Context, req *operationPb.CowsAnalysisRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.MixFeedStatisticsParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.StartTime,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchAccuracyAggStatistics 准确性分析-汇总分析
func (s *StoreEntry) SearchAccuracyAggStatistics(ctx context.Context, req *operationPb.AccuracyAggStatisticsRequest) (*model.PastureCommonResponse, error) {
	body := &model.FeedChartParams{
		ParamMaps: &model.AccuracyAggParams{
			PastureId: fmt.Sprintf("%d", req.PastureId),
			StartTime: req.StartTime,
			StopTime:  req.EndTime,
			FName:     req.Fname,
			Sort:      req.Sort,
			Status:    req.Status,
			Times:     req.Times,
			Genre:     req.Genre,
			IsDate:    req.Isdate,
			Hlwc1:     req.Hlwc1,
			Hlwc2:     req.Hlwc2,
			Hlzq1:     req.Hlzq1,
			Hlzq2:     req.Hlzq2,
			Hlzql1:    req.Hlzql1,
			Hlzql2:    req.Hlzql2,
			Slwc1:     req.Slwc1,
			Slwc2:     req.Slwc2,
			Slzq1:     req.Slzq1,
			Slzq2:     req.Slzq2,
			Slzql1:    req.Slzql1,
			Slzql2:    req.Slzql2,
			Error:     req.IsError,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlSummary, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchMixFeedStatistics 准确性分析-混料统计
func (s *StoreEntry) SearchMixFeedStatistics(ctx context.Context, req *operationPb.MixFeedStatisticsRequest) (*model.PastureCommonResponse, error) {
	times := ""
	if req.ClassNumber > 0 {
		times = fmt.Sprintf("%d", req.ClassNumber)
	}
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.MixFeedStatisticsParams{
			PastureId:   fmt.Sprintf("%d", req.PastureId),
			StartTime:   req.StartTime,
			StopTime:    req.StartTime,
			TmrTName:    req.EquipmentName,
			ProjName:    req.TrainNumber,
			Times:       times,
			ButtonType:  req.ButtonType,
			TempletName: req.FormulationName,
			Isuse:       req.IsUse,
			Hlwc1:       req.Hlwc1,
			Hlwc2:       req.Hlwc2,
			Hlzq1:       req.Hlzq1,
			Hlzq2:       req.Hlzq2,
			Hlzql1:      req.Hlzql1,
			Hlzql2:      req.Hlzql2,
			Error:       req.IsError,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchSprinkleStatistics 准确性分析-撒料统计
func (s *StoreEntry) SearchSprinkleStatistics(ctx context.Context, req *operationPb.SprinkleStatisticsRequest) (*model.PastureCommonResponse, error) {
	times := ""
	if req.ClassNumber > 0 {
		times = fmt.Sprintf("%d", req.ClassNumber)
	}
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.SprinkleStatisticsParams{
			PastureId:   fmt.Sprintf("%d", req.PastureId),
			StartTime:   req.StartTime,
			StopTime:    req.StartTime,
			TmrTName:    req.EquipmentName,
			ProjName:    req.TrainNumber,
			Times:       times,
			ButtonType:  req.ButtonType,
			TempletName: req.FormulationName,
			Isuse:       req.IsUse,
			Fname:       req.BarnName,
			Slwc1:       req.Slwc1,
			Slwc2:       req.Slwc2,
			Slzq2:       req.Slzq2,
			Slzq1:       req.Slzq1,
			Slzql1:      req.Slzql1,
			Slzql2:      req.Slzql2,
			Error:       req.IsError,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlDataByName, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

// SearchProcessAnalysis 过程分析
func (s *StoreEntry) SearchProcessAnalysis(ctx context.Context, req *operationPb.ProcessAnalysisRequest) (*model.PastureCommonResponse, error) {
	body := &model.PastureCommonRequest{
		Name:       req.ApiName,
		Page:       req.Pagination.Page,
		Offset:     req.Pagination.PageOffset,
		PageCount:  req.Pagination.PageSize,
		ReturnType: "Map",
		ParamMaps: &model.ProcessAnalysisParams{
			PastureId:   fmt.Sprintf("%d", req.PastureId),
			StartTime:   req.StartTime,
			StopTime:    req.StartTime,
			TmrTName:    req.TmrName,
			IsCompleted: "",
			LpPlanType:  fmt.Sprintf("%d", req.PlanType),
			FClassId:    req.MixFeedType,
			Hlzq1:       req.Hlzq1,
			Hlzq2:       req.Hlzq2,
			Hlwc1:       req.Hlwc1,
			Hlwc2:       req.Hlwc2,
			Slwc1:       req.Slwc1,
			Slwc2:       req.Slwc2,
			Slzq2:       req.Slzq2,
			Slzq1:       req.Slzq1,
			Error:       req.ErrorRange,
		},
	}
	response := &model.PastureCommonResponse{Data: &model.PastureCommonData{}}
	if err := s.PastureHttpClient(ctx, model.UrlProcess, int64(req.PastureId), body, response); err != nil {
		return nil, xerr.WithStack(err)
	}
	return response, nil
}

func (s *StoreEntry) SearchAnalysisAccuracy(ctx context.Context, req *operationPb.SearchAnalysisAccuracyRequest) (*model.SearchAnalysisAccuracyResponse, error) {

	dataList := &model.CommonValueRatio{
		MaxValue:    "",
		MiddleValue: "",
		MinValue:    "",
		PastureName: make([]string, 0),
		DateDay:     make([]string, 0),
		DataList:    make([][]string, 0),
		PastureIds:  make([]int32, 0),
	}
	res := &model.SearchAnalysisAccuracyResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &model.AnalysisAccuracyData{
			Chart: &model.Chart{
				MixedFodderAccurateRatio:    dataList,
				MixedFodderCorrectRatio:     dataList,
				SprinkleFodderAccurateRatio: dataList,
				SprinkleFodderCorrectRatio:  dataList,
			},
			Table: &model.Table{
				TitleList: make([]*model.TableList, 0),
				DataList:  make([]*interface{}, 0),
			},
		},
	}
	res.Data.Table.TitleList = append(res.Data.Table.TitleList, &model.TableList{
		Name:  "title",
		Value: "牧场",
	})

	analysisAccuracy := make([]*model.OptionsAnalysisAccuracy, 0)
	pref := s.DB.Model(new(model.AnalysisAccuracy))
	if req.EndDate != "" && req.StartDate != "" {
		pref.Where("date_day BETWEEN ? AND ?", req.StartDate, req.EndDate)
	}

	if req.CattleParentCategoryId > 0 {
		pref.Where("cattle_parent_category_id = ?", req.CattleParentCategoryId)
	}
	if req.FeedFormulaId > 0 {
		pref.Where("feed_formula_id = ?", req.FeedFormulaId)
	}

	if len(req.PastureIds) > 0 {
		pref.Where("pasture_id IN ?", req.PastureIds)
	}

	if err := pref.Select("pasture_id,pasture_name,date_day,sum(iweight) as all_iweight,sum(lweight) as all_lweight,sum(oweight) as all_oweight,sum(actual_weight_minus) as all_actual_weight_minus,sum(allow_ratio) as all_allow_ratio,sum(alweight) as all_alweight").
		Group("pasture_id,pasture_name,date_day").Order("pasture_name,date_day").Find(&analysisAccuracy).Debug().Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	mixedFodderAccurateRatio := make([]float64, 0)
	mapPastureName := make(map[string]bool, 0)
	mapDateDay := make(map[string]bool, 0)
	mapRatio := make(map[string]bool, 0)
	tableList := make([]*model.TableList, 0)
	for k, v := range analysisAccuracy {
		if _, ok := mapPastureName[v.PastureName]; !ok {
			res.Data.Chart.MixedFodderAccurateRatio.PastureName = append(res.Data.Chart.MixedFodderAccurateRatio.PastureName, v.PastureName)
			res.Data.Chart.MixedFodderAccurateRatio.PastureIds = append(res.Data.Chart.MixedFodderAccurateRatio.PastureIds, int32(v.PastureId))
			mapPastureName[v.PastureName] = true
		}

		dataDayFormat := strings.TrimRight(v.DateDay, "T00:00:00+08:00")
		if _, ok := mapDateDay[v.DateDay]; !ok {
			res.Data.Chart.MixedFodderAccurateRatio.DateDay = append(res.Data.Chart.MixedFodderAccurateRatio.DateDay, dataDayFormat)
			mapDateDay[v.DateDay] = true
		}

		valueRatio1 := float64(v.AllIweight/v.AllLweight) / 100.0
		if _, ok := mapRatio[v.DateDay]; !ok {
			valueRatio := make([]string, 0)
			for _, a := range analysisAccuracy {
				if a.DateDay == v.DateDay {
					valueRatio = append(valueRatio, strconv.FormatFloat(float64(a.AllIweight/a.AllLweight)/100.0, 'f', 2, 64))
				}
			}
			res.Data.Chart.MixedFodderAccurateRatio.DataList = append(res.Data.Chart.MixedFodderAccurateRatio.DataList, valueRatio)
			mapRatio[v.DateDay] = true
			tableList = append(tableList, &model.TableList{
				Name:  fmt.Sprintf("date%d", k),
				Value: dataDayFormat,
			})
		}
		mixedFodderAccurateRatio = append(mixedFodderAccurateRatio, valueRatio1)
	}

	mixedFodderAccurateRatioMaxValue, mixedFodderAccurateRatioMiddleValue, mixedFodderAccurateRatioMinValue := calculateRatio(mixedFodderAccurateRatio)
	res.Data.Chart.MixedFodderAccurateRatio.MaxValue = strconv.FormatFloat(mixedFodderAccurateRatioMaxValue, 'f', 2, 64)
	res.Data.Chart.MixedFodderAccurateRatio.MiddleValue = strconv.FormatFloat(mixedFodderAccurateRatioMiddleValue, 'f', 2, 64)
	res.Data.Chart.MixedFodderAccurateRatio.MinValue = strconv.FormatFloat(mixedFodderAccurateRatioMinValue, 'f', 2, 64)

	res.Data.Table.TitleList = append(res.Data.Table.TitleList, tableList...)

	return res, nil
}

func calculateRatio(res []float64) (float64, float64, float64) {
	var maxValue, middleValue, minValue, allValue float64 = 0, 0, 0, 0
	for _, v := range res {
		if v > maxValue {
			maxValue = v
		}
		if v <= maxValue {
			minValue = v
		}

		allValue += v
	}
	middleValue = allValue / float64(len(res))
	return maxValue, middleValue, minValue
}
